#include "i2c.h"

/******************************************************************************/
/*                            Local Macros                                    */
/******************************************************************************/
/* ----------------------------------------------------------------------------
|  Ensure I2C port 1 is idle:                                                  |
|                                                                              |
|  Active wait                                                                 |
 -----------------------------------------------------------------------------*/
#define _I2C1_CHECK_IDLE() while (I2C1CONbits.SEN || I2C1CONbits.PEN 	 \
				|| I2C1CONbits.RCEN || I2C1CONbits.ACKEN \
				|| I2C1STATbits.TRSTAT			 \
			)
#define MAX_CTO 1000
/******************************************************************************/
/*                         Global Variables                                   */
/******************************************************************************/
/* ... */
volatile unsigned char mao = 0;
/******************************************************************************/
/*                          Local Variables                                   */
/******************************************************************************/
/* static .... */
/******************************************************************************/
/*                          Function Definitions                              */
/******************************************************************************/
/* ----------------------------------------------------------------------------
|  Initialize I2C Port 1:                                                      |
|                                                                              |
|  TODO	                                                                       |
 ---------------------------------------------------------------------------- */
i2c_status_t
i2c_init(void)
{
	I2C1CONbits.I2CEN = 0;		/* Stop I2C port */

	IEC1bits.SI2C1IE = 0;		/* Disable Interrupts */
	IEC1bits.MI2C1IE = 0;		/* ... */
	IFS1bits.SI2C1IF = 0;		/* Clear Interrupt flag bits */
	IFS1bits.MI2C1IF = 0;		/* ... */

	ODCGbits.ODCG2=1;		/* Configre SCL/SDA pin as open-drain */
	ODCGbits.ODCG3=1;		/* ... */

	I2C1ADD=0;			/* Clear Address */
	I2C1MSK=0;			/* Clear mask */

	/* chris TODO: un-hardcode:
	 * use reg_val as param or use KHz as param
	 * and calculate here the reg_val
 	*/

	//I2C1BRG = 395;	/* Set baudrate: SCL=100KHz @ Fcy=40MHz */
	//I2C1BRG = 272;	/* Set baudrate: SCL=100KHz @ Fcy=40MHz */
	I2C1BRG = 362;		/* Set baudrate: SCL=100KHz @ Fcy=40MHz */

	I2C1CON = 0;			/* Configure I2C port */
	I2C1CONbits.ACKDT = 1;		/* chris TODO: ???? */
	//I2C1CONbits.DISSLW = 1;	/* chris TODO: ???? */

	I2C1CONbits.I2CEN = 1;		/* Start I2C port */

	return I2C_SUCCESS;
}


/* ----------------------------------------------------------------------------
|  Read from I2C Port 1:                                                       |
|                                                                              |
|  TODO	                                                                       |
 ---------------------------------------------------------------------------- */
i2c_status_t
i2c_read_byte(unsigned char device, unsigned char address, unsigned char *data)
{
	unsigned int cto = 0; // Time out counter for i2c

	_I2C1_CHECK_IDLE();

	/* Initiate Start on SDA and SCL pins */
	I2C1CONbits.SEN = 1;

	/* Wait till Start sequence is completed */
	while (I2C1CONbits.SEN)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	/* Write slave address, set master for transmission
	*  and check for write collision.
	*/
	/* TODO: check if device address is correct (R/W bit should be 0) */
	I2C1TRN = device;
	if (I2C1STATbits.IWCOL)
		return I2C_FAILURE;

	/* Wait till address is transmitted */
	while (I2C1STATbits.TBF)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	/* Test for ACK condition received:
	*  First check the InterruptFlag (work-around), then check ACK bit.
	*/
	IFS1bits.MI2C1IF=0;
	while (IFS1bits.MI2C1IF==0)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	while (I2C1STATbits.ACKSTAT)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	/* Write slave register address and check write collision. */
	I2C1TRN = address;

	if (I2C1STATbits.IWCOL)
		return I2C_FAILURE;

	/* Wait till address is transmitted */
	while (I2C1STATbits.TBF)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	/* Test for ACK condition received:
	*  First check the InterruptFlag (work-around), then check ACK bit.
	*/
	IFS1bits.MI2C1IF=0;
	while (IFS1bits.MI2C1IF==0)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	while (I2C1STATbits.ACKSTAT)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	/* Initiate re-start on SDA and SCL pins */
	I2C1CONbits.RSEN = 1;

	/* Wait until re-start condition is over */
	 while (I2C1CONbits.RSEN)
	 {
		 if(++cto > MAX_CTO)
			 return  I2C_FAILURE;

	 }

	cto = 0; //restart time-out

	/* Write Slave address, set master for reception  (R/W bit should be 1)
	*  and check for write collision
	*/
	/* TODO: check if device address is correct  (R/W bit should be 1)*/
	I2C1TRN = device+1;
	if (I2C1STATbits.IWCOL)
		return I2C_FAILURE;

	/* Wait till address is transmitted */
	while (I2C1STATbits.TBF)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	/* Test for ACK condition received:
	*  First check the InterruptFlag (work-around), then check ACK bit.
	*/
	IFS1bits.MI2C1IF=0;
	while (IFS1bits.MI2C1IF==0)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	while (I2C1STATbits.ACKSTAT)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	/* Read the data byte */
	I2C1CONbits.RCEN = 1;
	while (I2C1CONbits.RCEN)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	I2C1STATbits.I2COV = 0;
	*data = I2C1RCV & 0x00FF;

	_I2C1_CHECK_IDLE();

	/* send NACK condition back to the I2C slave indicating master
	* received the data byte
	*/
	I2C1CONbits.ACKDT = 1;
	I2C1CONbits.ACKEN = 1;
	while (I2C1CONbits.ACKEN) 	/* wait until NACK sequence is over */
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	/* Initiate Stop on SDA and SCL pins */
	I2C1CONbits.PEN = 1;

	/* Wait till Stop sequence is completed  */
	while (I2C1CONbits.PEN)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	return I2C_SUCCESS;
}


/* ----------------------------------------------------------------------------
|  Write to I2C Port 1:                                                        |
|                                                                              |
|  TODO	                                                                       |
 ---------------------------------------------------------------------------- */
i2c_status_t
i2c_write_byte(unsigned char device, unsigned char address, unsigned char data)
{
	unsigned int cto = 0; // Time out counter for i2c

	_I2C1_CHECK_IDLE();

	/* initiate Start on SDA and SCL pins */
	I2C1CONbits.SEN = 1;

	/* Wait till Start sequence is completed */
	while (I2C1CONbits.SEN)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;

	}

	cto = 0; //restart time-out

	/* Write slave address, set master for transmission
	*  and check for write collision.
	*/
	/* TODO: check if device address is correct (R/W bit should be 0) */
	I2C1TRN = device;
	if (I2C1STATbits.IWCOL)
		return I2C_FAILURE;

	/* Wait till address is transmitted */
	while (I2C1STATbits.TBF)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	/* Test for ACK condition received:
	*  First check the InterruptFlag (work-around), then check ACK bit.
	*/
	IFS1bits.MI2C1IF=0;
	while (IFS1bits.MI2C1IF==0)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	while (I2C1STATbits.ACKSTAT)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	/* Write slave register address and check write collision. */
	I2C1TRN = address;
	if (I2C1STATbits.IWCOL)
		return I2C_FAILURE;

	/* Wait till address is transmitted */
	while (I2C1STATbits.TBF)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	/* Test for ACK condition received:
	*  First check the InterruptFlag (work-around), then check ACK bit.
	*/
	IFS1bits.MI2C1IF=0;
	while (IFS1bits.MI2C1IF==0)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	while (I2C1STATbits.ACKSTAT)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	/* Write data byte to slave register and check write collision  */
	I2C1TRN = data;

	if (I2C1STATbits.IWCOL)
		return I2C_FAILURE;

	/* Wait till data is transmitted */
	while (I2C1STATbits.TBF)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	/* Test for ACK condition received:
	*  First check the InterruptFlag (work-around), then check ACK bit.
	*/
	IFS1bits.MI2C1IF=0;
	while (IFS1bits.MI2C1IF==0)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	while (I2C1STATbits.ACKSTAT)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();

	/* initiate Stop on SDA and SCL pins */
	I2C1CONbits.PEN = 1;

	/* Wait till Stop sequence is completed  */
	while (I2C1CONbits.PEN)
	{
		if(++cto > MAX_CTO)
			return  I2C_FAILURE;
	}

	cto = 0; //restart time-out

	_I2C1_CHECK_IDLE();
	/*IFS1bits.MI2C1IF=0;
	EE_led_on();*/
	return I2C_SUCCESS;
}



