/*
 * linesensor.c
 *
 *  Created on: Apr 21, 2009
 *      Author: Mangesh Chitnis
 */

#include "linesensor.h"
#define LINE_WIDTH 640

extern linesensor_mode_t ls_mode; //Line sensor operating mode

extern volatile unsigned char image[19600] __attribute__((far));  //One dimensional image
//extern unsigned char image[19600] __attribute__ ((space(auto_psv)));
volatile unsigned char backgroundline[LINE_WIDTH] __attribute__((far)); //Background line

//static unsigned int objectCNT = 0;
//static unsigned char objectST = 0;

/* ----------------------------------------------------------------------------
|  Check the line sensor mode of operation and take appropriate action         |
 ---------------------------------------------------------------------------- */
int linesensor_mode_check(){

	switch(ls_mode){
		case end_device_sends_line:
			return 1;
		case end_device_background_update:
			return linesensor_realtime_background();
		case end_device_sends_detected_lines:
			return linesensor_realtime_background();
		case end_device_sends_only_report:
			return linesensor_realtime_background();
			/*if(objectST){
				objectST = 0;
				return 1;
			}
			return 0;*/
		case end_device_sends_feature_vector:
			return 1;
		case end_device_does_image_classification:
			return 1;
		case end_device_sends_colored_lines:
			return 1;
		default :
			//linesensor_realtime_background();
			return 1;
	}
}


/* ----------------------------------------------------------------------------
|  Recognize the image background and store it in a background image.		   	|
|  Background is calculated using an average of 40 foreground free lines		|
|  todo: Update background using median operator and update with histogram based|
|  threshold																	|
 ---------------------------------------------------------------------------- */
int linesensor_realtime_background(){
	static int linecount = 0;
	int i;

	/*Use first 40 lines to calculate the background*/
	if(linecount >= 0 && linecount < 40){
		for(i=0; i < LINE_WIDTH - 1; i++){
			backgroundline[i] = (char)(((int)(backgroundline[i]+image[i]))/2);
		}
		linecount++;
		return 1;
	}
	else{
		return linesensor_background_update();
	}
}

int linesensor_background_update(){

	static unsigned char THRESHOLD = 25; /* Threshold to identify foreground */
	static unsigned char MIN_OBJ_DET_LINE_CNT = 6; /* Minimum number of lines to recognize an object */

	int thPixelcount = 0; /* Consecutive object pixels within a single line */
	static unsigned char objDetLinecount = 0; /* Detected object lines */

	int i;
	//D8_filterflag = true;
	//connected_labelflag = true;

	/*Update the background and identify foreground with a threhold*/
	for(i=0; i < LINE_WIDTH - 1; i++){
	  if(image[i] > backgroundline[i])
		  image[i] = image[i] - backgroundline[i];
	  else
		  image[i] = backgroundline[i] - image[i];

	  /* Binary object with white foreground and dark background*/
		if(image[i] > THRESHOLD && i < LINE_WIDTH - 10){
			image[i] = 255;
			thPixelcount++;
		}
		else{
			image[i] = 0;
			//check for the break in lines; this is to check continuos pixels with value 255
			//to detect and object line; all other lines are noise.
			if(thPixelcount < THRESHOLD)
				thPixelcount = 0;
		}
	}

	/*if(ls_mode == end_device_sends_detected_lines)
		return 0;*/
	/*
	if((ls_mode == end_device_sends_detected_lines ||
			ls_mode == end_device_sends_only_report) && (thPixelcount < THRESHOLD)){
		//Before making 0 check whether object was detected in previous line
		if(objDetLinecount > MIN_OBJ_DET_LINE_CNT){
			objectCNT++;
			objectST = 1;
		}
		objDetLinecount = 0;
		return 0;
	}else{
		objDetLinecount++;
		return 1;
	}*/

	/*Depending on operating mode return the processed line*/
	switch(ls_mode){
			case end_device_sends_line:
				return 1;

			case end_device_background_update:
				return 1;

			case end_device_sends_detected_lines:
				if(thPixelcount < THRESHOLD) return 0;
				else return 1;

			case end_device_sends_only_report:
				if(thPixelcount > THRESHOLD) {objDetLinecount++;}
				else{
					if(objDetLinecount > MIN_OBJ_DET_LINE_CNT){
						objDetLinecount = 0;
						return 1;
					}
					objDetLinecount = 0;
				}
				return 0;
			/*case end_device_sends_feature_vector:
				return 1;
			case end_device_does_image_classification:
				return 1;
			case end_device_sends_colored_lines:
				return 1;*/
			default :
				//linesensor_realtime_background();
				return 1;
		}
}

/*
void linesensor_connected_labelling(){

}

void linesensor_boundary_extraction(){

}*/

